package CXC::ACIS::Command::Dump::Tree;

use strict;
use warnings;

## no critic (ProhibitAccessOfPrivateData)

our $VERSION = '0.03';

use Pegex::Base;

use Carp;

extends 'Pegex::Tree';

use CXC::ACIS::Command::Constants qw[ %CMD ];

sub got_command_stream {

    croak( "internal error: too many command streams\n" )
      if @{$_[1]} != 1;

    return $_[1][0];

}

# compress a consecutive array of similarly named inner blocks into a
# single attribute whose value is an array of the block contents
sub got_element_blocks {

    my $blocks = $_[1];

    my @blocks = ( [ $blocks->[0][0], [] ] );

    for my $block ( @{$_[1]} ) {

	my ( $name, $values ) = @$block;

	push @blocks, [ $name, [] ]
	   unless $name eq $blocks[-1][0];

	push @{$blocks[-1][-1]}, $values;
    }

   return @blocks;

}

sub _array_to_hash {

    my $values = shift;

    my %values;

    for my $value ( @$values ) {

	my ( $name, $value ) = @$value;
	croak( "repeated attribute: $name\n" )
	  if exists $values{$name};

	$values{$name} = $value;
    }

    return \%values;
}


sub got_command_block {

    my ( $name, $id, $values ) = @{ $_[1] };

    $values = _array_to_hash( $values );

    croak( "$name\[$id]: missing commandOpcode parameter\n" )
      unless defined $values->{commandOpcode};

    my $cmd = $CMD{ $values->{commandOpcode} };
    croak( "unknown opcode: $values->{commandOpcode}\n" )
      unless defined $cmd;

    $values->{commandName} = $cmd->{name};

    return $values;

}


sub got_element_block {

    my ( $name, $id, $values ) = @{ $_[1] };

    $values = _array_to_hash( $values );

    return [ $name => $values ];

}


sub got_attribute {

    my ( $name, $value ) = @{$_[1]};

    # not possible to discern if attribute is a scalar or an array; hardwire
    # things so it works.


    unless ( $name =~ /
		        patchIds
		      | writeData
		      | functionArguments
		      | initialData
		      /x ) {

	$value = $value->[0] if @$value == 1;

    }

    return [ $name, $value ];

}

sub got_decint{ $_[1] + 0 };
sub got_hexint{ hex $_[1] };
