#! perl

use 5.10.1;

use strict;
use warnings;

use CXC::ACIS::Command::Block;

*bits = \&CXC::ACIS::Command::Block::bits;

use Test::More;


# input: starting offset in bits, length
#
#  Algorithm derived from Royce Buehler's lcmd bit routine
#
#  1. Find 16 bit word containing the offset, index = offset >> 4.
#
#  2. The LSB bit requested is bit = offset & 15, where bit = 0 is the
#     *rightmost* bit in the word
#
#  3. If more bits are required than are available to the left of the
#     LSB bit in the current word, take them from the *next* word,
#     starting at the rightmost bit.



my $pad = '1101111010101011';

for my $offset ( 0, 1, 2, 3, 4 ) {

    subtest "no straddle, must shift, offset => $offset" => sub {



        my $v = pack(
            'B*', $pad x $offset
              #    fedcba9876543210
              . qw[1110000000000000] . $pad
        );

        is( bits( $v, 0xd + $offset * 16, 3 ), 0b111 );
        is( bits( $v, 0xd + $offset * 16, 2 ),  0b11 );
        is( bits( $v, 0xd + $offset * 16, 1 ),   0b1 );

    };

    subtest "no straddle, no shift, offset => $offset" => sub {

        my $v = pack(
            'B*', $pad x $offset
              #    fedcba9876543210
              . qw[0000000000000111] . $pad
        );

        is( bits( $v, 0x00 + $offset * 16, 3 ), 0b111 );
        is( bits( $v, 0x00 + $offset * 16, 2 ),  0b11 );
        is( bits( $v, 0x00 + $offset * 16, 1 ),   0b1 );

    };

}

for my $offset ( 0, 1, 2, 3, 4 ) {

    subtest "straddle one word, must shift, offset => $offset" => sub {

        my $v = pack(
            'B*',
            $pad x $offset
              #    fedcba9876543210
              . qw[1110000000000000] . qw[1111000000010000] . $pad
        );

        is( bits( $v, 0xd + $offset * 16, 8 ),           0b10000111 );
        is( bits( $v, 0xd + $offset * 16, 16 ),  0b1000000010000111 );
        is( bits( $v, 0xd + $offset * 16, 17 ), 0b11000000010000111 );

    };

    subtest "straddle two words, must shift, offset => $offset" => sub {

        my $v = pack(
            'B*', $pad x $offset
              #    fedcba9876543210
              . qw[1110000000000000]
              . $pad
              . qw[1111000000010000]
              . $pad
        );


        is( bits( $v, 0xd + $offset * 16, 8 ),                  0b01011111 );
        is( bits( $v, 0xd + $offset * 16, 16 ),         0b1111010101011111 );
        is( bits( $v, 0xd + $offset * 16, 19 ),      0b1101111010101011111 );
        is( bits( $v, 0xd + $offset * 16, 24 ), 0b100001101111010101011111 );

    };

}


done_testing;
